<?php
/**
 * @file
 * Common functions available to user_generated_content module.
 */

/**
 * Validate form value is a node type
 *
 * @param string $node_type
 *   Form value from project node.
 *
 * @return boolean
 *   Valid node type.
 */
function user_generated_content_is_valid_node_type($node_type) {
  $types = node_get_types('types');
  return array_key_exists($node_type, $types);
}

/**
 * Validate UGC form submission.
 *
 * @param array $form
 *   Form definition.
 *
 * @param array $state
 *   Form submission values.
 */
function user_generated_content_build_form_validate($form, &$state) {
  // Do not validate if ahah submission.
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $errors = array();
  $node_type = $state['values']['ugc_form_node'];
  // Get field definitions.
  $result = db_query('SELECT * FROM {content_node_field_instance} AS cnfi JOIN {content_node_field} AS cnf ON cnfi.field_name = cnf.field_name WHERE cnfi.type_name = \'%s\' ORDER BY cnfi.weight ASC', $node_type);
  $fields = array();
  while ($f = db_fetch_object($result)) {
    $fields[] = $f;
  }
  // Validate title and body.
  if (empty($state['values']['title'])) {
    $errors['title'] = 'Title is a required field.';
  }
  if (empty($state['values']['body'])) {
    $errors['body'] = 'Body is a required field.';
  }
  // Validate fields.
  foreach ($fields as $f) {
    // Does field need creating as a hidden field.
    $display_settings = unserialize($f->display_settings);
    if ((int) $display_settings['full']['exclude'] == 1) {
      $f->widget_type = 'hidden';
    }
    // Only validate native supported fields.
    if (!user_generated_content_field_supported_natively($f)) {
      continue;
    }
    // Validate required field.
    if ($f->required == 1) {
      if (is_array($state['values'][$f->field_name])) {
        if (empty($state['values'][$f->field_name][0])) {
          $errors[$f->field_name . '][0'] = $f->label . ' is a requried field.';
        }
      }
      elseif (empty($state['values'][$f->field_name])) {
        $errors[$f->field_name] = $f->label . ' is a requried field.';
      }
    }
    switch ($f->widget_module) {
      // Validate a number field.
      case 'number':
        if (is_array($state['values'][$f->field_name])) {
          foreach ($state['values'][$f->field_name] as $key => $value) {
            if (!is_numeric($value) && !empty($value)) {
              $errors[$f->field_name . '][' . $key] = $f->label . ': Value must be numeric.';
            }
          }
        }
        elseif (!is_numeric($state['values'][$f->field_name]) && !empty($state['values'][$f->field_name])) {
          $errors[$f->field_name] = $f->label . ': Value must be numeric.';
        }
        break;
    }
  }
  // Validate taxonomy values.
  $ptax = array();
  $result = db_query('SELECT td.* FROM {term_node} AS tn JOIN {term_data} AS td ON tn.tid = td.tid WHERE tn.nid = %d ORDER BY td.weight ASC', $state['values']['ugc_project_id']);
  while ($t = db_fetch_object($result)) {
    if (empty($ptax[$t->vid])) {
      $ptax[$t->vid] = array();
    }
    $ptax[$t->vid][] = $t;
  }
  $ftax = array();
  $result = db_query('SELECT td.* FROM {vocabulary_node_types} AS vnt JOIN {term_data} AS td ON vnt.vid = td.vid WHERE vnt.type = \'%s\' ORDER BY td.weight ASC', $state['values']['ugc_form_node']);
  while ($t = db_fetch_object($result)) {
    if (!empty($ptax[$t->vid])) {
      $ftax[$t->vid] = $ptax[$t->vid];
    }
    else {
      if (empty($ftax[$t->vid])) {
        $ftax[$t->vid] = array();
      }
      $ftax[$t->vid][] = $t;
    }
  }
  foreach ($ftax as $vid => $terms) {
    // Get vocabulary record.
    $v = db_fetch_object(db_query('SELECT * FROM {vocabulary} WHERE vid = %d', $vid));
    if ($v->required == 1 && empty($state['values']['taxonomy'][$vid])) {
      $errors['taxonomy][' . $vid] = $v->name . ' is a requried field.';
    }
  }
  // Invoke module hooks for UGC validation.
  $tmp = module_invoke_all('ugc_form_validate_extension', $errors, $state, $state['values']['ugc_project_id']);
  if (is_array($tmp)) {
    $errors = array_merge($errors, $tmp);
  }
  // Output errors.
  if (!empty($errors) && is_array($errors)) {
    foreach ($errors as $field => $err) {
      form_set_error($field, check_plain($err));
    }
  }
}

/**
 * Process UGC form submission.
 *
 * @param array $form
 *   Form definition.
 * @param array $state
 *   Form submission values.
 */
function user_generated_content_build_form_submit($form, &$state) {
  // Do not process if ahah submission.
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $node_type = $state['values']['ugc_form_node'];
  // Get project node.
  $project = node_load($state['values']['ugc_project_id']);
  // Get field definitions.
  $result = db_query('SELECT * FROM {content_node_field_instance} AS cnfi JOIN {content_node_field} AS cnf ON cnfi.field_name = cnf.field_name WHERE cnfi.type_name = \'%s\' ORDER BY cnfi.weight ASC', $node_type);
  $fields = array();
  while ($f = db_fetch_object($result)) {
    $fields[] = $f;
  }
  $node = new stdClass();
  // Build node object.
  $node->revision = FALSE;
  $node->type = $node_type;
  $node->comment = $project->comment;
  $node->title = $state['values']['title'];
  $node->body = $state['values']['body'];
  // Add node fields.
  foreach ($fields as $f) {
    // Does field need creating as a hidden field.
    $display_settings = unserialize($f->display_settings);
    if ((int) $display_settings['full']['exclude'] == 1) {
      $f->widget_type = 'hidden';
    }
    // Only process native supported fields.
    if (!user_generated_content_field_supported_natively($f)) {
      continue;
    }
    if (isset($state['values'][$f->field_name])) {
      // Get field db column suffix.
      $db_column = unserialize($f->db_columns);
      $keys = array_keys($db_column);
      $suffix = (!empty($keys[0])) ? $keys[0] : 'value';
      // Add field values.
      if (is_array($state['values'][$f->field_name])) {
        foreach ($state['values'][$f->field_name] as $key => $value) {
          if ($f->widget_type == 'optionwidgets_buttons') {
            if (!empty($value)) {
              $node->{$f->field_name}[] = array($suffix => $key);
            }
          }
          else {
            $node->{$f->field_name}[] = array($suffix => $value);
          }
        }
      }
      else {
        $node->{$f->field_name}[0] = array($suffix => $state['values'][$f->field_name]);
      }
    }
  }
  // Add taxonomy values.
  $node->taxonomy = array();
  if (!empty($state['values']['taxonomy'])) {
    foreach ($state['values']['taxonomy'] as $vid => $tid) {
      if (!empty($tid)) {
        if (is_array($tid)) {
          foreach ($tid as $t) {
            if (!empty($t)) {
              $node->taxonomy[$t] = taxonomy_get_term($t);
            }
          }
        }
        else {
          $node->taxonomy[$tid] = taxonomy_get_term($tid);
        }
      }
    }
  }
  // Invoke module hooks for UGC submission.
  $tmp = module_invoke_all('ugc_form_save_extension', $node, $state, $state['values']['ugc_project_id']);
  if (!empty($tmp)) {
    $node = (is_array($tmp)) ? $tmp[0] : $tmp;
  }
  // Save node record.
  $node = node_submit($node);
  $node->uid = $GLOBALS['user']->uid;
  $node->status = (!empty($project->field_automatically_publish[0]['value'])) ? 1 : 0;
  node_save($node);
  // Add relationship to project record.
  $relation = new stdClass();
  $relation->ugc_project_node = $project->nid;
  $relation->ugc_content_node = $node->nid;
  drupal_write_record('ugc_project_content_nodes', $relation);
  // Send notification email.
  if (!empty($project->field_notification_email[0]['value'])) {
    $params = array(
      'title' => $node->title,
      'nid' => $node->nid,
      'status' => (($node->status) ? 'Published' : 'Unpublished'),
    );
    drupal_mail('user_generated_content', 'notify', $project->field_notification_email[0]['value'], user_preferred_language($GLOBALS['user']), $params);
  }
  // Set submission message.
  if (!empty($project->field_submission_message[0]['value'])) {
    drupal_set_message($project->field_submission_message[0]['value']);
  }
  else {
    drupal_set_message(t('Your content has been created.'));
  }
  // Invoke module hooks for UGC node saved.
  module_invoke_all('ugc_node_created', $node, $project->nid);
}

/**
 * Get field definitions to build UGC form.
 *
 * @param string $node_type
 *   Node type to use for form
 * @param int $project_nid
 *   The node id for the UGC project node
 * @param array $state
 *   Form state
 *
 * @return array
 *   Definition of form fields
 */
function user_generated_content_get_form_field_definition($node_type, $project_nid, $state) {
  // Get node type definition.
  $type = db_fetch_object(db_query('SELECT * FROM {node_type} WHERE type = \'%s\'', $node_type));
  $result = db_query('SELECT * FROM {content_node_field_instance} AS cnfi JOIN {content_node_field} AS cnf ON cnfi.field_name = cnf.field_name WHERE cnfi.type_name = \'%s\' ORDER BY cnfi.weight ASC', $node_type);
  $fields = array();
  while ($f = db_fetch_object($result)) {
    $fields[] = $f;
  }
  // Get taxonomies available to form.
  $ptax = array();
  $result = db_query('SELECT td.* FROM {term_node} AS tn JOIN {term_data} AS td ON tn.tid = td.tid WHERE tn.nid = %d ORDER BY td.weight ASC', $project_nid);
  while ($t = db_fetch_object($result)) {
    if (empty($ptax[$t->vid])) {
      $ptax[$t->vid] = array();
    }
    $ptax[$t->vid][] = $t;
  }
  $ftax = array();
  $result = db_query('SELECT td.* FROM {vocabulary_node_types} AS vnt JOIN {term_data} AS td ON td.vid = vnt.vid WHERE vnt.type = \'%s\' ORDER BY td.weight ASC', $node_type);
  while ($t = db_fetch_object($result)) {
    if (!empty($ptax[$t->vid])) {
      $ftax[$t->vid] = $ptax[$t->vid];
    }
    else {
      if (empty($ftax[$t->vid])) {
        $ftax[$t->vid] = array();
      }
      $ftax[$t->vid][] = $t;
    }
  }
  $extra_weights = variable_get('content_extra_weights_' . $node_type, array());
  // Build definition for each field.
  $definition = array();
  // Title field definition.
  if ($type->has_title) {
    $params = array(
      'label' => $type->title_label,
      'multiple' => 0,
      'required' => TRUE,
      'weight' => ((!empty($extra_weights['title'])) ? $extra_weights['title'] : 0),
    );
    $definition[] = user_generated_content_build_text_field('title', 'field', $params, $state);
  }
  // Body field definition.
  if ($type->has_body) {
    $params = array(
      'label' => $type->body_label,
      'multiple' => 0,
      'required' => TRUE,
      'weight' => ((!empty($extra_weights['body_field'])) ? $extra_weights['body_field'] : 0),
    );
    $definition[] = user_generated_content_build_text_field('body', 'area', $params, $state);
  }
  // Loop defined fields for node type.
  foreach ($fields as $f) {
    // Check for ugc_extended_build_XX_field function,
    if (function_exists('ugc_extended_build_' . $f->widget_type . '_field')) {
      $custom_field = call_user_func('ugc_extended_build_' . $f->widget_type . '_field', $f, $project_nid, $node_type, $state);
      if (!empty($custom_field)) {
        $definition[] = $custom_field;
      }
    }
    else {
      // Check widget type is supported by default.
      if (user_generated_content_field_type_supported($f->widget_type)) {
        // Does field need creating as a hidden field.
        $display_settings = unserialize($f->display_settings);
        if ((int) $display_settings['full']['exclude'] == 1) {
          $f->widget_type = 'hidden';
        }
        // Get field settings.
        $widget_settings = unserialize($f->widget_settings);
        $global_settings = unserialize($f->global_settings);
        // Get field default value.
        $default = '';
        // Build field definition based on widget type.
        switch ($f->widget_type) {
          case 'number':
          case 'text_textfield':
            $params = array(
              'label' => $f->label,
              'multiple' => $f->multiple,
              'required' => $f->required,
              'weight' => $f->weight,
              'description' => $f->description,
              'default' => $default,
              'maxlength' => (isset($global_settings['max_length'])) ? $global_settings['max_length'] : 0,
              'size' => (isset($widget_settings['size'])) ? $widget_settings['size'] : 0,
            );
            $definition[] = user_generated_content_build_text_field($f->field_name, 'field', $params, $state);
            break;

          case 'text_textarea':
            $params = array(
              'label' => $f->label,
              'multiple' => $f->multiple,
              'required' => $f->required,
              'weight' => $f->weight,
              'description' => $f->description,
              'default' => $default,
              'rows' => (isset($widget_settings['rows'])) ? $widget_settings['rows'] : 0,
            );
            $definition[] = user_generated_content_build_text_field($f->field_name, 'area', $params, $state);
            break;

          case 'optionwidgets_select':
            $params = array(
              'label' => $f->label,
              'multiple' => $f->multiple,
              'required' => $f->required,
              'weight' => $f->weight,
              'description' => $f->description,
              'default' => $default,
            );
            // Get select options.
            $options = user_generated_content_get_allowed_values($f->field_name, $global_settings);
            $definition[] = user_generated_content_build_options_field($f->field_name, 'select', $options, $params, $state);
            break;

          case 'optionwidgets_buttons':
            $params = array(
              'label' => $f->label,
              'multiple' => $f->multiple,
              'required' => $f->required,
              'weight' => $f->weight,
              'description' => $f->description,
              'default' => $default,
            );
            // Get select options.
            $options = user_generated_content_get_allowed_values($f->field_name, $global_settings);
            $definition[] = user_generated_content_build_options_field($f->field_name, 'button', $options, $params, $state);
            break;

          case 'optionwidgets_onoff':
            $params = array(
              'label' => $f->label,
              'required' => $f->required,
              'weight' => $f->weight,
              'description' => $f->description,
              'default' => $default,
            );
            // Get select options.
            $options = user_generated_content_get_allowed_values($f->field_name, $global_settings);
            $definition[] = user_generated_content_build_checkbox_field($f->field_name, $options, $params, $state);
            break;

          case 'hidden':
            $definition[] = user_generated_content_build_hidden_field($f->field_name, $default);
            break;
        }
      }
    }
  }
  // Add taxonomy fields.
  if (!empty($ftax)) {
    $definition[] = user_generated_content_build_taxonomy_field($ftax, (($extra_weights['taxonomy']) ? $extra_weights['taxonomy'] : 0));
  }
  // Add requried hidden fields.
  $definition[] = user_generated_content_build_hidden_field('ugc_project_id', $project_nid);
  $definition[] = user_generated_content_build_hidden_field('ugc_form_node', $node_type);
  // Add submit button.
  $definition[] = user_generated_content_build_button_field('create_content', array('label' => 'Create content', 'weight' => 1000));
  return $definition;
}

/**
 * Check if field type is supported as a UGC field
 *
 * @param string $field_type
 *   The field widget type defined in the node type definition.
 *
 * @return boolean
 *   Field support.
 */
function user_generated_content_field_type_supported($field_type) {
  $allowed_widgets = array(
    'number',
    'text_textfield',
    'text_textarea',
    'optionwidgets_select',
    'optionwidgets_buttons',
    'optionwidgets_onoff',
  );
  if (in_array($field_type, $allowed_widgets)) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Build a text field definition.
 *
 * @param string $name
 *   Field name.
 * @param string $type
 *   Field text type, field or area.
 * @param array $params
 *   Parameters for text field.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_text_field($name, $type, $params, $state) {
  $default_value = (!empty($params['default'])) ? $params['default'] : '';
  $field_type = (($type == 'area') ? 'textarea' : 'textfield');
  $weight = (!empty($params['weight'])) ? $params['weight'] : 0;
  $title = (!empty($params['label'])) ? $params['label'] : '';
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($name, $field_type);
  // If allowing multiple.
  if (!empty($params['multiple'])) {
    $field = array(
      $name => array(
        '#tree' => TRUE,
        '#title' => $title,
        '#weight' => $weight,
        '#theme' => 'ugc_multi_field',
      ),
    );
    // Add multiple entry fields.
    $num_fields = ($params['multiple'] == 1) ? 1 : (int) $params['multiple'];
    // Build number of fields based on form state.
    if (!empty($state['values'][$name])) {
      $num_fields = count($state['values'][$name]);
      if ($state['add_another_field'] == $name) {
        $num_fields++;
      }
    }
    for ($i = 0; $i < $num_fields; $i++) {
      $field[$name][$i] = array(
        '#type' => $field_type,
        '#default_value' => (!empty($state['values'][$name][$i])) ? $state['values'][$name][$i] : $default_value,
        '#weight' => $i,
        '#wysiwyg' => FALSE,
      );
      switch ($type) {
        case 'field':
          $field[$name][$i]['#size'] = (!empty($params['size'])) ? $params['size'] : 60;
          if (!empty($params['maxlength'])) {
            $field[$name][$i]['#maxlength'] = $params['maxlength'];
          }
          break;

        case 'area':
          $field[$name][$i]['#rows'] = (!empty($params['rows'])) ? $params['rows'] : 5;
          break;
      }
      if (!empty($theme)) {
        $field[$name][$i]['#theme'] = $theme;
      }
    }
    // If unlimited, add button for 'add another'.
    if ($params['multiple'] == 1) {
      // Ensure unique id for container div.
      $id = uniqid($name);
      $field[$name]['#prefix'] = '<div id="multi-wrap-' . $id . '">';
      $field[$name]['#suffix'] = '</div>';
      $field[$name]['add_another'] = array(
        '#type' => 'submit',
        '#tree' => FALSE,
        '#value' => 'Add another',
        '#weight' => ($weight + 1 + $num_fields),
        '#ahah' => array(
          'path' => 'ugc/ahah/add_another/' . $name,
          'wrapper' => 'multi-wrap-' . $id,
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
    }
  }
  else {
    $field = array(
      $name => array(
        '#type' => $field_type,
        '#title' => $title,
        '#default_value' => (!empty($state['values'][$name])) ? $state['values'][$name] : $default_value,
        '#weight' => $weight,
        '#wysiwyg' => FALSE,
      ),
    );
    if (!empty($params['description'])) {
      $field[$name]['#description'] = $param['description'];
    }
    switch ($type) {
      case 'field':
        $field[$name]['#size'] = (!empty($params['size'])) ? $params['size'] : 60;
        if (!empty($params['maxlength'])) {
          $field[$name]['#maxlength'] = $params['maxlength'];
        }
        break;

      case 'area':
        $field[$name]['#rows'] = (!empty($params['rows'])) ? $params['rows'] : 5;
        break;
    }
    if (!empty($theme)) {
      $field[$name]['#theme'] = $theme;
    }
  }
  return $field;
}

/**
 * Build a options field definition (select/checkbox/radio button).
 *
 * @param string $name
 *   Field name.
 * @param string $type
 *   Field type  select / button.
 * @param array $options
 *   Select options, value => text.
 * @param array $params
 *   Parameters for texy field.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_options_field($name, $type, $options, $params, $state) {
  // Set field type.
  $field_type = $type;
  if ($type == 'button' && !empty($params['multiple'])) {
    $field_type = 'checkboxes';
  }
  elseif ($type == 'button') {
    $field_type = 'radios';
  }
  // Set default value.
  $default = (!empty($params['default'])) ? $params['default'] : '';
  $deafult = (!empty($state['values'][$name])) ? $state['values'][$name] : $default;
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($name, $field_type);
  // Define field.
  $field = array(
    $name => array(
      '#type' => $field_type,
      '#title' => (!empty($params['label'])) ? $params['label'] : '',
      '#options' => $options,
      '#default' => $default,
      '#weight' => (!empty($params['weight'])) ? $params['weight'] : 0,
    ),
  );
  if (!empty($params['multiple']) && $type == 'select') {
    $field[$name]['#multiple'] = TRUE;
    $field[$name]['#size'] = 5;
  }
  if (!empty($params['description'])) {
    $field[$name]['#description'] = $params['description'];
  }
  if (!empty($theme)) {
    $field[$name]['#theme'] = $theme;
  }
  return $field;
}

/**
 * Build a single checkbox field definition.
 *
 * @param string $name
 *   Field name.
 * @param array $options
 *   Select options, value => text.
 * @param array $params
 *   Parameters for texy field.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_checkbox_field($name, $options, $params, $state) {
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($name, 'checkbox');
  // Get selected value.
  $keys = array_keys($options);
  $on_value = (!empty($keys) && isset($keys[1])) ? $keys[1] : NULL;
  // Set default value.
  $default = (!empty($params['default'])) ? $params['default'] == $on_value : FALSE;
  $default = (!empty($state['values'][$name])) ? $state['values'][$name] == $on_Value : $default;
  // Define field.
  $field = array(
    $name => array(
      '#type' => 'checkbox',
      '#title' => (!empty($options[$on_value])) ? $options[$on_value] : $params['label'],
      '#default_value' => $default,
      '#weight' => (!empty($params['weight'])) ? $params['weight'] : 0,
      '#return_value' => $on_value,
    ),
  );
  if (!empty($params['description'])) {
    $field[$name]['#description'] = $params['description'];
  }
  if (!empty($theme)) {
    $field[$name]['#theme'] = $theme;
  }
  return $field;
}

/**
 * Build a text field definition.
 *
 * @param string $name
 *   Field name.
 * @param string $default
 *   Default value.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_hidden_field($name, $default) {
  $field = array(
    $name => array(
      '#type' => 'hidden',
      '#default_value' => $default,
      '#weight' => 1000,
    ),
  );
  return $field;
}

/**
 * Build a button field definition.
 *
 * @param string $name
 *   Field name.
 * @param array $params
 *   Extra parameters
 *   format - array(
 *     'label' - button label
 *     'weight' - field ordering weight
 *     'validate' - define a validation function
 *     'submit' - define a submit function
 *     'ahah' - define an ahah callback
 *   ).
 *
 * @return array
 *   Definition of field
 */
function user_generated_content_build_button_field($name, $params) {
  $field = array(
    $name => array(
      '#type' => 'submit',
      '#value' => $params['label'],
    ),
  );
  // Add weight parameter.
  if (isset($params['weight'])) {
    $field[$name]['#weight'] = $params['weight'];
  }
  // Add validate function.
  if (!empty($params['validate'])) {
    $field[$name]['#validate'] = array($params['validate']);
  }
  // Add submit function.
  if (!empty($params['submit'])) {
    $field[$name]['#submit'] = array($params['submit']);
  }
  // Add ahah element.
  if (!empty($params['ahah'])) {
    $field[$name]['#ahah'] = $params['ahah'];
  }
  return $field;
}

/**
 * Build a taxonomy field definition.
 *
 * @param array $vocabs
 *   Vocab and taxnomies available - format array[vid] => (taxonomy object).
 * @param integer $weight
 *   Field order weight.
 *
 * @return array
 *   Definition of field.
 */
function user_generated_content_build_taxonomy_field($vocabs, $weight) {
  $field = array(
    'taxonomy' => array(
      '#tree' => TRUE,
      '#title' => 'Taxonomies',
      '#weight' => (!empty($weight)) ? $weight : 0,
      '#theme' => 'ugc_field_taxonomy',
    ),
  );
  // Add each vocabulary field.
  foreach ($vocabs as $vid => $terms) {
    // Get vocabulary record.
    $v = db_fetch_object(db_query('SELECT * FROM {vocabulary} WHERE vid = %d', $vid));
    // Build options array.
    $options = array('' => 'Select..');
    foreach ($terms as $term) {
      $options[$term->tid] = $term->name;
    }
    // Create field element.
    $field['taxonomy'][$vid] = array(
      '#type' => 'select',
      '#title' => $v->name,
      '#options' => $options,
      '#default_value' => ($v->multiple) ? array() : '',
    );
    if ($v->multiple) {
      $field['taxonomy'][$vid]['#multiple'] = TRUE;
    }
    if (!empty($v->description)) {
      $field['taxonomy'][$vid]['#description'] = $v->description;
    }
    // Get theme function.
    $theme = user_generated_content_get_field_theme_function('vocabulary_' . $vid, 'select');
    if (!empty($theme)) {
      $field['taxonomy'][$vid]['#theme'] = $theme;
    }
  }
  return $field;
}

/**
 * Get theme function name to apply to a field.
 *
 * @param string $name
 *   Field name.
 * @param string $type
 *   Form api field type.
 *
 * @return string
 *   Function name, NULL if no theme function exists.
 */
function user_generated_content_get_field_theme_function($name, $type) {
  $functions = array('ugc_field_' . $name, 'ugc_field_' . $type);
  foreach ($functions as $function) {
    if (user_generated_content_theme_function_exists($function)) {
      return $function;
    }
  }
  return NULL;
}

/**
 * Check theme function exists.
 *
 * $param string $function
 *   Name of function to check.
 *
 * @return boolean
 *   Function exists
 */
function user_generated_content_theme_function_exists($function) {
  global $theme, $theme_engine;
  if (empty($theme)) {
    return FALSE;
  }
  if (function_exists($theme . '_' . $function)) {
    return TRUE;
  }
  elseif (!empty($theme_engine) && function_exists($theme_engine . '_' . $function)) {
    return TRUE;
  }
  elseif (function_exists('theme_' . $function)) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Get allowed values for field.
 *
 * @param string $name
 *   The field name.
 * @param array $global_settings
 *   The global settings value for the field.
 *
 * @return array
 *   Options.
 */
function user_generated_content_get_allowed_values($name, $global_settings) {
  $global_settings['field_name'] = $name;
  $options = content_allowed_values($global_settings, TRUE);
  return $options;
}

/**
 * Add another field via AHAH request.
 */
function user_generated_content_ahah_add_another() {
  $form_state = array(
    'storage' => NULL,
    'submitted' => FALSE,
    'rebuild' => TRUE,
  );
  $form_build_id = check_plain($_POST['form_build_id']);
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  // Add field name to add another of.
  $form_state['add_another_field'] = arg(3);
  // Enable the submit/validate handlers to determine whether AHAH-submittted.
  $form_state['ahah_submission'] = TRUE;
  $form['#programmed'] = $form['#redirect'] = FALSE;
  // Maintain form action, prevents error if form errors on submission.
  $form_state['action'] = $form['#action'];
  drupal_process_form($form_id, $form, $form_state);
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);
  // Get field to render.
  $field = $form[arg(3)];
  // Remove the prefix/suffix wrapper so we don't double it up.
  unset($field['#prefix'], $field['#suffix']);
  // Render the output.
  $output = theme('status_messages');
  $output .= drupal_render($field);
  // Get header javascript.
  $javascript = drupal_add_js(NULL, NULL, 'header');
  // Rendering callback.
  drupal_set_header('Content-Type: text/html;');
  $response = array(
    'status' => TRUE,
    'data' => $output,
    'settings' => call_user_func_array('array_merge_recursive', $javascript['setting']),
  );
  echo drupal_to_js($response);
  exit();
}
